package geniusweb.references;

import java.util.HashMap;

/**
 * init parameters for a party. Object must be either a HashMap, List, String or
 * Number. Other objects may not deserialize properly. We never use blanket
 * Object deserializers (enableDefaultTyping) because of security reasons.
 *
 */
public class Parameters extends HashMap<String, Object> {

	/**
	 * Getter with type-check
	 * 
	 * @param <T>       the expected type
	 * @param paramname the parameter name that must be in the map
	 * @param classType the expected value type for this parameter
	 * @return object of requested type.
	 * @throws IllegalArgumentException if the object is not available.
	 */
	@SuppressWarnings("unchecked")
	public <T> T get(String paramname, Class<T> classType) {
		if (!containsKey(paramname) || !(classType.isInstance(get(paramname))))
			throw new IllegalArgumentException(" Missing a parameter "
					+ paramname + " with a " + classType.getName() + " value");
		return (T) get(paramname);
	}

	/**
	 * 
	 * @param name       the parameter name
	 * @param defaultval the default value to return if parameter is not
	 *                   available or not inside the given range. Can be null
	 * @param min        the minimum value, or null if not applicable
	 * @param max        the maximum value, or null if not applicable
	 * @return the double value contained inside the parameter with the given
	 *         name, or defaultval if the parameter does not exist, does not
	 *         contain a double, or is outside [min, max].
	 */
	public Double getDouble(String name, Double defaultval, Double min,
			Double max) {
		if (!containsKey(name) || !(get(name) instanceof Double))
			return defaultval;
		Double val = (Double) get(name);
		if (min != null && val < min)
			return defaultval;
		if (max != null && val > max)
			return defaultval;
		return val;
	}
}
